/*
 *
 * fminbnd.h
 *
 * Code generation for function 'fminbnd'
 *
 */

#ifndef FMINBND_H
#define FMINBND_H

/* Include files */
#include <stddef.h>
#include <stdlib.h>
#include "rtwtypes.h"
#include "omp.h"
#include "fetch_thresholds_types.h"

/* Function Declarations */
extern double fminbnd(const cell_wrap_2 funfcnInput_tunableEnvironment[3],
                      double ax, double bx);

#endif

/* End of code generation (fminbnd.h) */
