/*
 *
 * std.h
 *
 * Code generation for function 'std'
 *
 */

#ifndef STD_H
#define STD_H

/* Include files */
#include <stddef.h>
#include <stdlib.h>
#include "rtwtypes.h"
#include "omp.h"
#include "fetch_thresholds_types.h"

/* Function Declarations */
extern double b_std(const emxArray_real_T *x);

#endif

/* End of code generation (std.h) */
