/*
 *
 * rtGetInf.h
 *
 * Code generation for function 'fetch_thresholds'
 *
 */

#ifndef RTGETINF_H
#define RTGETINF_H
#include "rtwtypes.h"
#include "rt_nonfinite.h"
#ifdef __cplusplus

extern "C" {

#endif

  extern real_T rtGetInf(void);
  extern real32_T rtGetInfF(void);
  extern real_T rtGetMinusInf(void);
  extern real32_T rtGetMinusInfF(void);

#ifdef __cplusplus

}
#endif
#endif

/* End of code generation (rtGetInf.h) */
