/*
 *
 * histcounts.h
 *
 * Code generation for function 'histcounts'
 *
 */

#ifndef HISTCOUNTS_H
#define HISTCOUNTS_H

/* Include files */
#include <stddef.h>
#include <stdlib.h>
#include "rtwtypes.h"
#include "omp.h"
#include "fetch_thresholds_types.h"

/* Function Declarations */
extern void histcounts(const emxArray_real_T *x, double varargin_1,
  emxArray_real_T *n, emxArray_real_T *edges);

#endif

/* End of code generation (histcounts.h) */
