/*
 *
 * fetch_thresholds_initialize.h
 *
 * Code generation for function 'fetch_thresholds_initialize'
 *
 */

#ifndef FETCH_THRESHOLDS_INITIALIZE_H
#define FETCH_THRESHOLDS_INITIALIZE_H

/* Include files */
#include <stddef.h>
#include <stdlib.h>
#include "rtwtypes.h"
#include "omp.h"
#include "fetch_thresholds_types.h"

/* Function Declarations */
extern void fetch_thresholds_initialize(void);

#endif

/* End of code generation (fetch_thresholds_initialize.h) */
